@@ -29,35 +29,36 @@
 
 /******************************************************************************/
 
-const messaging = vAPI.messaging;
-const cmEditor = new CodeMirror(
-    document.getElementById('userFilters'),
-    {
-        autofocus: true,
-        lineNumbers: true,
-        lineWrapping: true,
-        styleActiveLine: true
-    }
-);
+const cmEditor = new CodeMirror(document.getElementById('userFilters'), {
+    autoCloseBrackets: true,
+    autofocus: true,
+    extraKeys: {
+        'Ctrl-Space': 'autocomplete',
+        'Tab': 'toggleComment',
+    },
+    foldGutter: true,
+    gutters: [ 'CodeMirror-linenumbers', 'CodeMirror-foldgutter' ],
+    lineNumbers: true,
+    lineWrapping: true,
+    matchBrackets: true,
+    maxScanLines: 1,
+    styleActiveLine: true,
+});
 
 uBlockDashboard.patchCodeMirrorEditor(cmEditor);
 
-let cachedUserFilters = '';
-
-/******************************************************************************/
-
-// https://github.com/gorhill/uBlock/issues/3706
-//   Save/restore cursor position
-//
-// CoreMirror reference: https://codemirror.net/doc/manual.html#api_selection
-
-window.addEventListener('beforeunload', ( ) => {
-    vAPI.localStorage.setItem(
-        'myFiltersCursorPosition',
-        JSON.stringify(cmEditor.getCursor().line)
-    );
+vAPI.messaging.send('dashboard', {
+    what: 'getAutoCompleteDetails'
+}).then(response => {
+    if ( response instanceof Object === false ) { return; }
+    const mode = cmEditor.getMode();
+    if ( mode.setHints instanceof Function ) {
+        mode.setHints(response);
+    }
 });
 
+let cachedUserFilters = '';
+
 /******************************************************************************/
 
 // This is to give a visual hint that the content of user blacklist has changed.
@@ -72,30 +73,20 @@ const userFiltersChanged = function(changed) {
 
 /******************************************************************************/
 
-const renderUserFilters = function(first) {
-    const onRead = function(details) {
-        if ( details.error ) { return; }
-        let content = details.content.trim();
-        cachedUserFilters = content;
-        if ( content.length !== 0 ) {
-            content += '\n';
-        }
-        cmEditor.setValue(content);
-        if ( first ) {
-            cmEditor.clearHistory();
-            try {
-                const line = JSON.parse(
-                    vAPI.localStorage.getItem('myFiltersCursorPosition')
-                );
-                if ( typeof line === 'number' ) {
-                    cmEditor.setCursor(line, 0);
-                }
-            } catch(ex) {
-            }
-        }
-        userFiltersChanged(false);
-    };
-    messaging.send('dashboard', { what: 'readUserFilters' }, onRead);
+const renderUserFilters = async function() {
+    const details = await vAPI.messaging.send('dashboard', {
+        what: 'readUserFilters',
+    });
+    if ( details instanceof Object === false || details.error ) { return; }
+
+    let content = details.content.trim();
+    cachedUserFilters = content;
+    if ( content.length !== 0 ) {
+        content += '\n';
+    }
+    cmEditor.setValue(content);
+
+    userFiltersChanged(false);
 };
 
 /******************************************************************************/
@@ -170,20 +161,18 @@ const exportUserFiltersToFile = function() {
 
 /******************************************************************************/
 
-const applyChanges = function() {
-    messaging.send(
-        'dashboard',
-        {
-            what: 'writeUserFilters',
-            content: cmEditor.getValue()
-        },
-        details => {
-            if ( details.error ) { return; }
-            cachedUserFilters = details.content.trim();
-            userFiltersChanged(false);
-            messaging.send('dashboard', { what: 'reloadAllFilters' });
-        }
-    );
+const applyChanges = async function() {
+    const details = await vAPI.messaging.send('dashboard', {
+        what: 'writeUserFilters',
+        content: cmEditor.getValue(),
+    });
+    if ( details instanceof Object === false || details.error ) { return; }
+
+    cachedUserFilters = details.content.trim();
+    userFiltersChanged(false);
+    vAPI.messaging.send('dashboard', {
+        what: 'reloadAllFilters',
+    });
 };
 
 const revertChanges = function() {
@@ -223,10 +212,35 @@ self.hasUnsavedData = function() {
 uDom('#importUserFiltersFromFile').on('click', startImportFilePicker);
 uDom('#importFilePicker').on('change', handleImportFilePicker);
 uDom('#exportUserFiltersToFile').on('click', exportUserFiltersToFile);
-uDom('#userFiltersApply').on('click', applyChanges);
+uDom('#userFiltersApply').on('click', ( ) => { applyChanges(); });
 uDom('#userFiltersRevert').on('click', revertChanges);
 
-renderUserFilters(true);
+// https://github.com/gorhill/uBlock/issues/3706
+//   Save/restore cursor position
+//
+// CodeMirror reference: https://codemirror.net/doc/manual.html#api_selection
+{
+    let curline = 0;
+    let timer;
+
+    renderUserFilters().then(( ) => {
+        cmEditor.clearHistory();
+        return vAPI.localStorage.getItemAsync('myFiltersCursorPosition');
+    }).then(line => {
+        if ( typeof line === 'number' ) {
+            cmEditor.setCursor(line, 0);
+        }
+        cmEditor.on('cursorActivity', ( ) => {
+            if ( timer !== undefined ) { return; }
+            if ( cmEditor.getCursor().line === curline ) { return; }
+            timer = vAPI.setTimeout(( ) => {
+                timer = undefined;
+                curline = cmEditor.getCursor().line;
+                vAPI.localStorage.setItem('myFiltersCursorPosition', curline);
+            }, 701);
+        });
+    });
+}
 
 cmEditor.on('changes', userFiltersChanged);
 CodeMirror.commands.save = applyChanges;
