@@ -34,11 +34,13 @@ const reValidExternalList = /[a-z-]+:\/\/\S*\/\S+/;
 
 let listDetails = {};
 let filteringSettingsHash = '';
-let hideUnusedSet = new Set();
+let hideUnusedSet = new Set([ '*' ]);
 
 /******************************************************************************/
 
-const onMessage = function(msg) {
+const messaging = vAPI.messaging;
+
+vAPI.broadcastListener.add(msg => {
     switch ( msg.what ) {
     case 'assetUpdated':
         updateAssetStatus(msg);
@@ -53,10 +55,7 @@ const onMessage = function(msg) {
     default:
         break;
     }
-};
-
-const messaging = vAPI.messaging;
-messaging.addChannelListener('dashboard', onMessage);
+});
 
 /******************************************************************************/
 
@@ -87,15 +86,17 @@ const renderFilterLists = function(soft) {
             li = listEntryTemplate.clone().nodeAt(0);
         }
         const on = entry.off !== true;
+        li.classList.toggle('checked', on);
         let elem;
         if ( li.getAttribute('data-listkey') !== listKey ) {
             li.setAttribute('data-listkey', listKey);
             elem = li.querySelector('input[type="checkbox"]');
             elem.checked = on;
-            elem = li.querySelector('a:nth-of-type(1)');
-            elem.setAttribute('href', 'asset-viewer.html?url=' + encodeURI(listKey));
-            elem.setAttribute('type', 'text/html');
+            elem = li.querySelector('.listname');
             elem.textContent = listNameFromListKey(listKey);
+            elem = li.querySelector('a.content');
+            elem.setAttribute('href', 'asset-viewer.html?url=' + encodeURIComponent(listKey));
+            elem.setAttribute('type', 'text/html');
             li.classList.remove('toRemove');
             if ( entry.supportName ) {
                 li.classList.add('support');
@@ -259,10 +260,6 @@ const renderFilterLists = function(soft) {
             let groupKey = groupKeys[i];
             let liGroup = liFromListGroup(groupKey, groups.get(groupKey));
             liGroup.setAttribute('data-groupkey', groupKey);
-            liGroup.classList.toggle(
-                'collapsed',
-                vAPI.localStorage.getItem('collapseGroup' + (i + 1)) === 'y'
-            );
             if ( liGroup.parentElement === null ) {
                 ulLists.appendChild(liGroup);
             }
@@ -306,23 +303,33 @@ const renderFilterLists = function(soft) {
         renderWidgets();
     };
 
-    messaging.send('dashboard', { what: 'getLists' }, onListsReceived);
+    messaging.send('dashboard', {
+        what: 'getLists',
+    }).then(details => {
+        onListsReceived(details);
+    });
 };
 
 /******************************************************************************/
 
 const renderWidgets = function() {
-    uDom('#buttonApply').toggleClass(
+    let cl = uDom.nodeFromId('buttonApply').classList;
+    cl.toggle(
         'disabled',
         filteringSettingsHash === hashFromCurrentFromSettings()
     );
-    uDom('#buttonPurgeAll').toggleClass(
+    const updating = document.body.classList.contains('updating');
+    cl = uDom.nodeFromId('buttonUpdate').classList;
+    cl.toggle('active', updating);
+    cl.toggle(
         'disabled',
-        document.querySelector('#lists .listEntry.cached:not(.obsolete)') === null
-    );
-    uDom('#buttonUpdate').toggleClass(
+        updating === false &&
+        document.querySelector('#lists .listEntry.obsolete:not(.toRemove) input[type="checkbox"]:checked') === null
+        );
+    cl = uDom.nodeFromId('buttonPurgeAll').classList;
+    cl.toggle(
         'disabled',
-        document.querySelector('body:not(.updating) #lists .listEntry.obsolete > input[type="checkbox"]:checked') === null
+        updating || document.querySelector('#lists .listEntry.cached:not(.obsolete)') === null
     );
 };
 
@@ -378,6 +385,15 @@ const hashFromCurrentFromSettings = function() {
 
 /******************************************************************************/
 
+const onListsetChanged = function(ev) {
+    const input = ev.target;
+    const li = input.closest('.listEntry');
+    li.classList.toggle('checked', input.checked);
+    onFilteringSettingsChanged();
+};
+
+/******************************************************************************/
+
 const onFilteringSettingsChanged = function() {
     renderWidgets();
 };
@@ -385,57 +401,58 @@ const onFilteringSettingsChanged = function() {
 /******************************************************************************/
 
 const onRemoveExternalList = function(ev) {
-    const liEntry = uDom(this).ancestors('[data-listkey]');
-    const listKey = liEntry.attr('data-listkey');
-    if ( listKey ) {
-        liEntry.toggleClass('toRemove');
-        renderWidgets();
-    }
-    ev.preventDefault();
+    const liEntry = ev.target.closest('[data-listkey]');
+    if ( liEntry === null ) { return; }
+    liEntry.classList.toggle('toRemove');
+    renderWidgets();
 };
 
 /******************************************************************************/
 
 const onPurgeClicked = function(ev) {
-    const button = uDom(ev.target);
-    const liEntry = button.ancestors('[data-listkey]');
-    const listKey = liEntry.attr('data-listkey');
-    if ( !listKey ) { return; }
+    const liEntry = ev.target.closest('[data-listkey]');
+    const listKey = liEntry.getAttribute('data-listkey') || '';
+    if ( listKey === '' ) { return; }
 
-    messaging.send('dashboard', { what: 'purgeCache', assetKey: listKey });
+    messaging.send('dashboard', {
+        what: 'purgeCache',
+        assetKey: listKey,
+    });
 
     // If the cached version is purged, the installed version must be assumed
     // to be obsolete.
     // https://github.com/gorhill/uBlock/issues/1733
     //   An external filter list must not be marked as obsolete, they will
     //   always be fetched anyways if there is no cached copy.
-    liEntry.addClass('obsolete');
-    liEntry.removeClass('cached');
+    liEntry.classList.add('obsolete');
+    liEntry.classList.remove('cached');
 
-    if ( liEntry.descendants('input').first().prop('checked') ) {
+    if ( liEntry.querySelector('input[type="checkbox"]').checked ) {
         renderWidgets();
     }
 };
 
 /******************************************************************************/
 
-const selectFilterLists = function(callback) {
+const selectFilterLists = async function() {
     // Cosmetic filtering switch
     messaging.send('dashboard', {
         what: 'userSettings',
         name: 'parseAllABPHideFilters',
-        value: document.getElementById('parseCosmeticFilters').checked
+        value: uDom.nodeFromId('parseCosmeticFilters').checked,
     });
     messaging.send('dashboard', {
         what: 'userSettings',
         name: 'ignoreGenericCosmeticFilters',
-        value: document.getElementById('ignoreGenericCosmeticFilters').checked
+        value: uDom.nodeFromId('ignoreGenericCosmeticFilters').checked,
     });
 
     // Filter lists to select
     const toSelect = [];
-    let liEntries = document.querySelectorAll('#lists .listEntry[data-listkey]:not(.toRemove)');
-    for ( const liEntry of liEntries ) {
+    for (
+        const liEntry of
+        document.querySelectorAll('#lists .listEntry[data-listkey]:not(.toRemove)')
+    ) {
         if ( liEntry.querySelector('input[type="checkbox"]:checked') !== null ) {
             toSelect.push(liEntry.getAttribute('data-listkey'));
         }
@@ -443,78 +460,70 @@ const selectFilterLists = function(callback) {
 
     // External filter lists to remove
     const toRemove = [];
-    liEntries = document.querySelectorAll('#lists .listEntry.toRemove[data-listkey]');
-    for ( const liEntry of liEntries ) {
+    for (
+        const liEntry of
+        document.querySelectorAll('#lists .listEntry.toRemove[data-listkey]')
+    ) {
         toRemove.push(liEntry.getAttribute('data-listkey'));
     }
 
     // External filter lists to import
     const externalListsElem = document.getElementById('externalLists');
     const toImport = externalListsElem.value.trim();
-    externalListsElem.value = '';
-    uDom.nodeFromId('importLists').checked = false;
+    {
+        const liEntry = externalListsElem.closest('.listEntry');
+        liEntry.classList.remove('checked');
+        liEntry.querySelector('input[type="checkbox"]').checked = false;
+        externalListsElem.value = '';
+    }
+
+    await messaging.send('dashboard', {
+        what: 'applyFilterListSelection',
+        toSelect: toSelect,
+        toImport: toImport,
+        toRemove: toRemove,
+    });
 
-    messaging.send(
-        'dashboard',
-        {
-            what: 'applyFilterListSelection',
-            toSelect: toSelect,
-            toImport: toImport,
-            toRemove: toRemove
-        },
-        callback
-    );
     filteringSettingsHash = hashFromCurrentFromSettings();
 };
 
 /******************************************************************************/
 
-const buttonApplyHandler = function() {
+const buttonApplyHandler = async function() {
     uDom('#buttonApply').removeClass('enabled');
-    selectFilterLists(( ) => {
-        messaging.send('dashboard', { what: 'reloadAllFilters' });
-    });
+    await selectFilterLists();
     renderWidgets();
+    messaging.send('dashboard', { what: 'reloadAllFilters' });
 };
 
 /******************************************************************************/
 
-const buttonUpdateHandler = function() {
-    selectFilterLists(( ) => {
-        document.body.classList.add('updating');
-        messaging.send('dashboard', { what: 'forceUpdateAssets' });
-        renderWidgets();
-    });
+const buttonUpdateHandler = async function() {
+    await selectFilterLists();
+    document.body.classList.add('updating');
     renderWidgets();
+    messaging.send('dashboard', { what: 'forceUpdateAssets' });
 };
 
 /******************************************************************************/
 
-const buttonPurgeAllHandler = function(ev) {
+const buttonPurgeAllHandler = async function(hard) {
     uDom('#buttonPurgeAll').removeClass('enabled');
-    messaging.send(
-        'dashboard',
-        {
-            what: 'purgeAllCaches',
-            hard: ev.ctrlKey && ev.shiftKey
-        },
-        ( ) => {
-            renderFilterLists(true);
-        }
-    );
+    await messaging.send('dashboard', {
+        what: 'purgeAllCaches',
+        hard,
+    });
+    renderFilterLists(true);
 };
 
 /******************************************************************************/
 
 const autoUpdateCheckboxChanged = function() {
-    messaging.send(
-        'dashboard',
-        {
-            what: 'userSettings',
-            name: 'autoUpdate',
-            value: this.checked
-        }
-    );
+    messaging.send('dashboard', {
+        what: 'userSettings',
+        name: 'autoUpdate',
+        value: this.checked,
+    });
 };
 
 /******************************************************************************/
@@ -551,17 +560,17 @@ const toggleHideUnusedLists = function(which) {
         groupSelector = '.groupEntry[data-groupkey="' + which + '"] ';
         uDom(groupSelector).toggleClass('hideUnused', mustHide);
     }
-    uDom(groupSelector + '.listEntry > input[type="checkbox"]:not(:checked)')
+    uDom(groupSelector + '.listEntry input[type="checkbox"]:not(:checked)')
         .ancestors('.listEntry[data-listkey]')
         .toggleClass('unused', mustHide);
     vAPI.localStorage.setItem(
         'hideUnusedFilterLists',
-        JSON.stringify(Array.from(hideUnusedSet))
+        Array.from(hideUnusedSet)
     );
 };
 
 const revealHiddenUsedLists = function() {
-    uDom('#lists .listEntry.unused > input[type="checkbox"]:checked')
+    uDom('#lists .listEntry.unused input[type="checkbox"]:checked')
         .ancestors('.listEntry[data-listkey]')
         .removeClass('unused');
 };
@@ -579,20 +588,11 @@ uDom('#lists').on('click', '.groupEntry[data-groupkey] > .geDetails', function(e
 });
 
 // Initialize from saved state.
-{
-    let aa;
-    try {
-        const json = vAPI.localStorage.getItem('hideUnusedFilterLists');
-        if ( json !== null ) {
-            aa = JSON.parse(json);
-        }
-    } catch (ex) {
+vAPI.localStorage.getItemAsync('hideUnusedFilterLists').then(value => {
+    if ( Array.isArray(value) ) {
+        hideUnusedSet = new Set(value);
     }
-    if ( Array.isArray(aa) === false ) {
-        aa = [ '*' ];
-    }
-    hideUnusedSet = new Set(aa);
-}
+});
 
 /******************************************************************************/
 
@@ -675,14 +675,21 @@ self.hasUnsavedData = function() {
 uDom('#autoUpdate').on('change', autoUpdateCheckboxChanged);
 uDom('#parseCosmeticFilters').on('change', onFilteringSettingsChanged);
 uDom('#ignoreGenericCosmeticFilters').on('change', onFilteringSettingsChanged);
-uDom('#buttonApply').on('click', buttonApplyHandler);
-uDom('#buttonUpdate').on('click', buttonUpdateHandler);
-uDom('#buttonPurgeAll').on('click', buttonPurgeAllHandler);
-uDom('#lists').on('change', '.listEntry > input', onFilteringSettingsChanged);
-uDom('#lists').on('click', '.listEntry > a.remove', onRemoveExternalList);
+uDom('#buttonApply').on('click', ( ) => { buttonApplyHandler(); });
+uDom('#buttonUpdate').on('click', ( ) => { buttonUpdateHandler(); });
+uDom('#buttonPurgeAll').on('click', ev => {
+    buttonPurgeAllHandler(ev.ctrlKey && ev.shiftKey);
+});
+uDom('#lists').on('change', '.listEntry input', onListsetChanged);
+uDom('#lists').on('click', '.listEntry .remove', onRemoveExternalList);
 uDom('#lists').on('click', 'span.cache', onPurgeClicked);
 uDom('#externalLists').on('input', onFilteringSettingsChanged);
 
+uDom('#lists').on('click', '.listEntry label *', ev => {
+    if ( ev.target.matches('a,input,.forinput') ) { return; }
+    ev.preventDefault();
+});
+
 /******************************************************************************/
 
 renderFilterLists();
