@@ -41,10 +41,6 @@ const µb = µBlock;
 
 /******************************************************************************/
 
-// To mitigate memory churning
-const netFilteringCacheJunkyard = [];
-const netFilteringCacheJunkyardMax = 10;
-
 const NetFilteringResultCache = class {
     constructor() {
         this.init();
@@ -58,14 +54,6 @@ const NetFilteringResultCache = class {
         return this;
     }
 
-    dispose() {
-        this.empty();
-        if ( netFilteringCacheJunkyard.length < netFilteringCacheJunkyardMax ) {
-            netFilteringCacheJunkyard.push(this);
-        }
-        return null;
-    }
-
     rememberResult(fctxt, result) {
         if ( fctxt.tabId <= 0 ) { return; }
         if ( this.results.size === 0 ) {
@@ -155,10 +143,7 @@ const NetFilteringResultCache = class {
     }
 
     static factory() {
-        const entry = netFilteringCacheJunkyard.pop();
-        return entry !== undefined
-            ? entry.init()
-            : new NetFilteringResultCache();
+        return new NetFilteringResultCache();
     }
 };
 
@@ -166,10 +151,7 @@ NetFilteringResultCache.prototype.shelfLife = 15000;
 
 /******************************************************************************/
 
-// Frame stores are used solely to associate a URL with a frame id. The
-// name `pageHostname` is used because of historical reasons. A more
-// appropriate name is `frameHostname` -- something to do in a future
-// refactoring.
+// Frame stores are used solely to associate a URL with a frame id.
 
 // To mitigate memory churning
 const frameStoreJunkyard = [];
@@ -181,15 +163,20 @@ const FrameStore = class {
     }
 
     init(frameURL) {
-        const µburi = µb.URI;
-        this.pageHostname = µburi.hostnameFromURI(frameURL);
-        this.pageDomain =
-            µburi.domainFromHostname(this.pageHostname) || this.pageHostname;
+        this.t0 = Date.now();
+        this.exceptCname = undefined;
+        this.rawURL = frameURL;
+        if ( frameURL !== undefined ) {
+            this.hostname = vAPI.hostnameFromURI(frameURL);
+            this.domain =
+                vAPI.domainFromHostname(this.hostname) || this.hostname;
+        }
         return this;
     }
 
     dispose() {
-        this.pageHostname = this.pageDomain = '';
+        this.exceptCname = undefined;
+        this.rawURL = this.hostname = this.domain = '';
         if ( frameStoreJunkyard.length < frameStoreJunkyardMax ) {
             frameStoreJunkyard.push(this);
         }
@@ -218,6 +205,7 @@ const PageStore = class {
         this.journalTimer = null;
         this.journalLastCommitted = this.journalLastUncommitted = undefined;
         this.journalLastUncommittedURL = undefined;
+        this.netFilteringCache = NetFilteringResultCache.factory();
         this.init(tabId, context);
     }
 
@@ -253,20 +241,20 @@ const PageStore = class {
         this.rawURL = tabContext.rawURL;
         this.hostnameToCountMap = new Map();
         this.contentLastModified = 0;
-        this.frames = new Map();
         this.logData = undefined;
         this.perLoadBlockedRequestCount = 0;
         this.perLoadAllowedRequestCount = 0;
-        this.hiddenElementCount = ''; // Empty string means "unknown"
         this.remoteFontCount = 0;
-        this.scriptCount = 0;
         this.popupBlockedCount = 0;
         this.largeMediaCount = 0;
         this.largeMediaTimer = null;
-        this.netFilteringCache = NetFilteringResultCache.factory();
         this.internalRedirectionCount = 0;
         this.extraData.clear();
 
+        this.frameAddCount = 0;
+        this.frames = new Map();
+        this.setFrame(0, tabContext.rawURL);
+
         // The current filtering context is cloned because:
         // - We may be called with or without the current context having been
         //   initialized.
@@ -320,6 +308,7 @@ const PageStore = class {
             // As part of https://github.com/chrisaljoudi/uBlock/issues/405
             // URL changed, force a re-evaluation of filtering switch
             this.rawURL = tabContext.rawURL;
+            this.setFrame(0, this.rawURL);
             return this;
         }
 
@@ -329,7 +318,6 @@ const PageStore = class {
             this.largeMediaTimer = null;
         }
         this.disposeFrameStores();
-        this.netFilteringCache = this.netFilteringCache.dispose();
         this.init(this.tabId, context);
         return this;
     }
@@ -339,13 +327,13 @@ const PageStore = class {
         this.title = '';
         this.rawURL = '';
         this.hostnameToCountMap = null;
+        this.netFilteringCache.empty();
         this.allowLargeMediaElementsUntil = 0;
         if ( this.largeMediaTimer !== null ) {
             clearTimeout(this.largeMediaTimer);
             this.largeMediaTimer = null;
         }
         this.disposeFrameStores();
-        this.netFilteringCache = this.netFilteringCache.dispose();
         if ( this.journalTimer !== null ) {
             clearTimeout(this.journalTimer);
             this.journalTimer = null;
@@ -373,8 +361,39 @@ const PageStore = class {
         const frameStore = this.frames.get(frameId);
         if ( frameStore !== undefined ) {
             frameStore.init(frameURL);
-        } else {
-            this.frames.set(frameId, FrameStore.factory(frameURL));
+            return;
+        }
+        this.frames.set(frameId, FrameStore.factory(frameURL));
+        this.frameAddCount += 1;
+        if ( (this.frameAddCount & 0b111111) !== 0 ) { return; }
+        this.pruneFrames();
+    }
+
+    // There is no event to tell us a specific subframe has been removed from
+    // the main document. The code below will remove subframes which are no
+    // longer present in the root document. Removing obsolete subframes is
+    // not a critical task, so this is executed just once on a while, to avoid
+    // bloated dictionary of subframes.
+    // A TTL is used to avoid race conditions when new iframes are added
+    // through the webRequest API but still not yet visible through the
+    // webNavigation API.
+    async pruneFrames() {
+        let entries;
+        try {
+            entries = await webext.webNavigation.getAllFrames({
+                tabId: this.tabId
+            });
+        } catch(ex) {
+        }
+        if ( Array.isArray(entries) === false ) { return; }
+        const toKeep = new Set();
+        for ( const { frameId } of entries ) {
+            toKeep.add(frameId);
+        }
+        const obsolete = Date.now() - 60000;
+        for ( const [ frameId, { t0 } ] of this.frames ) {
+            if ( toKeep.has(frameId) || t0 >= obsolete ) { continue; }
+            this.frames.delete(frameId);
         }
     }
 
@@ -394,7 +413,7 @@ const PageStore = class {
     }
 
     injectLargeMediaElementScriptlet() {
-        vAPI.tabs.injectScript(this.tabId, {
+        vAPI.tabs.executeScript(this.tabId, {
             file: '/js/scriptlets/load-large-media-interactive.js',
             allFrames: true,
             runAt: 'document_idle',
@@ -559,9 +578,22 @@ const PageStore = class {
 
         // Static filtering has lowest precedence.
         if ( result === 0 || result === 3 ) {
-            result = µb.staticNetFilteringEngine.matchString(fctxt);
-            if ( result !== 0 && µb.logger.enabled ) {
-                fctxt.filter = µb.staticNetFilteringEngine.toLogData();
+            const snfe = µb.staticNetFilteringEngine;
+            result = snfe.matchString(fctxt);
+            if ( result !== 0 ) {
+                if ( µb.logger.enabled ) {
+                    fctxt.filter = snfe.toLogData();
+                }
+                // https://github.com/uBlockOrigin/uBlock-issues/issues/943
+                //   Blanket-except blocked aliased canonical hostnames?
+                if (
+                    result === 1 &&
+                    fctxt.aliasURL !== undefined &&
+                    snfe.isBlockImportant() === false &&
+                    this.shouldExceptCname(fctxt)
+                ) {
+                    return 2;
+                }
             }
         }
 
@@ -664,6 +696,40 @@ const PageStore = class {
         return 1;
     }
 
+    shouldExceptCname(fctxt) {
+        let exceptCname;
+        let frameStore;
+        if ( fctxt.docId !== undefined ) {
+            frameStore = this.getFrame(fctxt.docId);
+            if ( frameStore instanceof Object ) {
+                exceptCname = frameStore.exceptCname;
+            }
+        }
+        if ( exceptCname === undefined ) {
+            const result = µb.staticNetFilteringEngine.matchStringReverse(
+                'cname',
+                frameStore instanceof Object
+                    ? frameStore.rawURL
+                    : fctxt.getDocOrigin()
+            );
+            if ( result === 2 ) {
+                exceptCname = µb.logger.enabled
+                    ? µb.staticNetFilteringEngine.toLogData()
+                    : true;
+            } else {
+                exceptCname = false;
+            }
+            if ( frameStore instanceof Object ) {
+                frameStore.exceptCname = exceptCname;
+            }
+        }
+        if ( exceptCname === false ) { return false; }
+        if ( exceptCname instanceof Object ) {
+            fctxt.setFilter(exceptCname);
+        }
+        return true;
+    }
+
     getBlockedResources(request, response) {
         const normalURL = µb.normalizePageURL(this.tabId, request.frameURL);
         const resources = request.resources;
