@@ -24,14 +24,16 @@
 /******************************************************************************/
 
 µBlock.scriptletFilteringEngine = (function() {
-    const µb = µBlock,
-        duplicates = new Set(),
-        scriptletCache = new µb.MRUCache(32),
-        reEscapeScriptArg = /[\\'"]/g;
+    const µb = µBlock;
+    const duplicates = new Set();
+    const scriptletCache = new µb.MRUCache(32);
+    const reEscapeScriptArg = /[\\'"]/g;
 
-    let acceptedCount = 0,
-        discardedCount = 0,
-        scriptletDB = new µb.staticExtFilteringEngine.HostnameBasedDB(1);
+    const scriptletDB = new µb.staticExtFilteringEngine.HostnameBasedDB(1);
+    const sessionScriptletDB = new µb.staticExtFilteringEngine.SessionDB();
+
+    let acceptedCount = 0;
+    let discardedCount = 0;
 
     const api = {
         get acceptedCount() {
@@ -169,15 +171,20 @@
         };
     })();
 
+    // TODO: Probably should move this into StaticFilteringParser
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1031
+    //   Normalize scriptlet name to its canonical, unaliased name.
     const normalizeRawFilter = function(rawFilter) {
-        let rawToken = rawFilter.slice(4, -1);
-        let rawEnd = rawToken.length;
+        const rawToken = rawFilter.slice(4, -1);
+        const rawEnd = rawToken.length;
         let end = rawToken.indexOf(',');
-        if ( end === -1 ) {
-            end = rawEnd;
-        }
-        let token = rawToken.slice(0, end).trim();
-        let normalized = token.endsWith('.js') ? token.slice(0, -3) : token;
+        if ( end === -1 ) { end = rawEnd; }
+        const token = rawToken.slice(0, end).trim();
+        const alias = token.endsWith('.js') ? token.slice(0, -3) : token;
+        let normalized = µb.redirectEngine.aliases.get(`${alias}.js`);
+        normalized = normalized === undefined
+            ? alias
+            : normalized.slice(0, -3);
         let beg = end + 1;
         while ( beg < rawEnd ) {
             end = rawToken.indexOf(',', beg);
@@ -280,20 +287,19 @@
         scriptletDB.collectGarbage();
     };
 
-    api.compile = function(parsed, writer) {
+    api.compile = function(parser, writer) {
         // 1001 = scriptlet injection
         writer.select(1001);
 
         // Only exception filters are allowed to be global.
-        const normalized = normalizeRawFilter(parsed.suffix);
+        const { raw, exception } = parser.result;
+        const normalized = normalizeRawFilter(raw);
 
         // Tokenless is meaningful only for exception filters.
-        if ( normalized === '+js()' && parsed.exception === false ) {
-            return;
-        }
+        if ( normalized === '+js()' && exception === false ) { return; }
 
-        if ( parsed.hostnames.length === 0 ) {
-            if ( parsed.exception ) {
+        if ( parser.hasOptions() === false ) {
+            if ( exception ) {
                 writer.push([ 32, '', 1, normalized ]);
             }
             return;
@@ -303,16 +309,13 @@
         //   Ignore instances of exception filter with negated hostnames,
         //   because there is no way to create an exception to an exception.
 
-        for ( let hn of parsed.hostnames ) {
-            const negated = hn.charCodeAt(0) === 0x7E /* '~' */;
-            if ( negated ) {
-                hn = hn.slice(1);
-            }
+        for ( const { hn, not, bad } of parser.extOptions() ) {
+            if ( bad ) { continue; }
             let kind = 0;
-            if ( parsed.exception ) {
-                if ( negated ) { continue; }
+            if ( exception ) {
+                if ( not ) { continue; }
                 kind |= 1;
-            } else if ( negated ) {
+            } else if ( not ) {
                 kind |= 1;
             }
             writer.push([ 32, hn, kind, normalized ]);
@@ -342,6 +345,14 @@
         }
     };
 
+    api.getSession = function() {
+        return sessionScriptletDB;
+    };
+
+    const $scriptlets = new Set();
+    const $exceptions = new Set();
+    const $scriptletToCodeMap = new Map();
+
     api.retrieve = function(request) {
         if ( scriptletDB.size === 0 ) { return; }
         if ( µb.hiddenSettings.ignoreScriptInjectFilters ) { return; }
@@ -360,41 +371,41 @@
             return;
         }
 
-        const scriptlets = new Set();
-        const exceptions = new Set();
+        $scriptlets.clear();
+        $exceptions.clear();
 
-        scriptletDB.retrieve(
-            hostname,
-            [ scriptlets, exceptions ]
-        );
+        if ( sessionScriptletDB.isNotEmpty ) {
+            sessionScriptletDB.retrieve([ null, $exceptions ]);
+        }
+        scriptletDB.retrieve(hostname, [ $scriptlets, $exceptions ]);
         if ( request.entity !== '' ) {
             scriptletDB.retrieve(
-                `${hostname.slice(0, -request.domain)}${request.entity}`,
-                [ scriptlets, exceptions ]
+                `${hostname.slice(0, -request.domain.length)}${request.entity}`,
+                [ $scriptlets, $exceptions ]
             );
         }
-        if ( scriptlets.size === 0 ) { return; }
+        if ( $scriptlets.size === 0 ) { return; }
 
         const loggerEnabled = µb.logger.enabled;
 
         // Wholly disable scriptlet injection?
-        if ( exceptions.has('') ) {
+        if ( $exceptions.has('') ) {
             if ( loggerEnabled ) {
                 logOne(true, '', request);
             }
             return;
         }
 
-        const scriptletToCodeMap = new Map();
-        for ( const rawToken of scriptlets ) {
-            lookupScriptlet(rawToken, reng, scriptletToCodeMap);
+        $scriptletToCodeMap.clear();
+        for ( const rawToken of $scriptlets ) {
+            lookupScriptlet(rawToken, reng, $scriptletToCodeMap);
         }
-        if ( scriptletToCodeMap.size === 0 ) { return; }
+        if ( $scriptletToCodeMap.size === 0 ) { return; }
 
         // Return an array of scriptlets, and log results if needed.
         const out = [];
-        for ( const [ rawToken, code ] of scriptletToCodeMap ) {
-            const isException = exceptions.has(rawToken);
+        for ( const [ rawToken, code ] of $scriptletToCodeMap ) {
+            const isException = $exceptions.has(rawToken);
             if ( isException === false ) {
                 out.push(code);
             }
@@ -445,15 +456,12 @@
         if ( µb.hiddenSettings.debugScriptletInjector ) {
             code = 'debugger;\n' + code;
         }
-        vAPI.tabs.injectScript(
-            details.tabId,
-            {
-                code: code,
-                frameId: details.frameId,
-                matchAboutBlank: false,
-                runAt: 'document_start'
-            }
-        );
+        vAPI.tabs.executeScript(details.tabId, {
+            code: code,
+            frameId: details.frameId,
+            matchAboutBlank: false,
+            runAt: 'document_start'
+        });
     };
 
     api.toSelfie = function() {
@@ -461,16 +469,16 @@
     };
 
     api.fromSelfie = function(selfie) {
-        scriptletDB = new µb.staticExtFilteringEngine.HostnameBasedDB(1, selfie);
+        scriptletDB.fromSelfie(selfie);
     };
 
     api.benchmark = async function() {
         const requests = await µb.loadBenchmarkDataset();
         if ( Array.isArray(requests) === false || requests.length === 0 ) {
-            console.info('No requests found to benchmark');
+            log.print('No requests found to benchmark');
             return;
         }
-        console.info('Benchmarking scriptletFilteringEngine.retrieve()...');
+        log.print('Benchmarking scriptletFilteringEngine.retrieve()...');
         const details = {
             domain: '',
             entity: '',
@@ -492,8 +500,8 @@
         }
         const t1 = self.performance.now();
         const dur = t1 - t0;
-        console.info(`Evaluated ${count} requests in ${dur.toFixed(0)} ms`);
-        console.info(`\tAverage: ${(dur / count).toFixed(3)} ms per request`);
+        log.print(`Evaluated ${count} requests in ${dur.toFixed(0)} ms`);
+        log.print(`\tAverage: ${(dur / count).toFixed(3)} ms per request`);
     };
 
     return api;
