@@ -1,7 +1,7 @@
 /*******************************************************************************
 
     uBlock Origin - a browser extension to block requests.
-    Copyright (C) 2014-2017 Raymond Hill
+    Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,10 +29,6 @@
 
 /******************************************************************************/
 
-const messaging = vAPI.messaging;
-
-/******************************************************************************/
-
 const handleImportFilePicker = function() {
     const file = this.files[0];
     if ( file === undefined || file.name === '' ) { return; }
@@ -74,16 +70,12 @@ const handleImportFilePicker = function() {
         const msg = vAPI.i18n('aboutRestoreDataConfirm')
                         .replace('{{time}}', time.toLocaleString());
         const proceed = window.confirm(msg);
-        if ( proceed ) {
-            messaging.send(
-                'dashboard',
-                {
-                    what: 'restoreUserData',
-                    userData: userData,
-                    file: filename
-                }
-            );
-        }
+        if ( proceed !== true ) { return; }
+        vAPI.messaging.send('dashboard', {
+            what: 'restoreUserData',
+            userData,
+            file: filename,
+        });
     };
 
     const fr = new FileReader();
@@ -104,33 +96,50 @@ const startImportFilePicker = function() {
 
 /******************************************************************************/
 
-const exportToFile = function() {
-    messaging.send('dashboard', { what: 'backupUserData' }, response => {
-        if (
-            response instanceof Object === false ||
-            response.userData instanceof Object === false
-        ) {
-            return;
-        }
-        vAPI.download({
-            'url': 'data:text/plain;charset=utf-8,' +
-                   encodeURIComponent(JSON.stringify(response.userData, null, '  ')),
-            'filename': response.localData.lastBackupFile
-        });
-        onLocalDataReceived(response.localData);
+const exportToFile = async function() {
+    const response = await vAPI.messaging.send('dashboard', {
+        what: 'backupUserData',
     });
+    if (
+        response instanceof Object === false ||
+        response.userData instanceof Object === false
+    ) {
+        return;
+    }
+    vAPI.download({
+        'url': 'data:text/plain;charset=utf-8,' +
+               encodeURIComponent(JSON.stringify(response.userData, null, '  ')),
+        'filename': response.localData.lastBackupFile
+    });
+    onLocalDataReceived(response.localData);
 };
 
 /******************************************************************************/
 
 const onLocalDataReceived = function(details) {
-    uDom('#localData > ul > li:nth-of-type(1)').text(
-        vAPI.i18n('settingsStorageUsed')
-            .replace(
-                '{{value}}',
-                typeof details.storageUsed === 'number' ? details.storageUsed.toLocaleString() : '?'
-            )
-    );
+    let v, unit;
+    if ( typeof details.storageUsed === 'number' ) {
+        v = details.storageUsed;
+        if ( v < 1e3 ) {
+            unit = 'genericBytes';
+        } else if ( v < 1e6 ) {
+            v /= 1e3;
+            unit = 'KB';
+        } else if ( v < 1e9 ) {
+            v /= 1e6;
+            unit = 'MB';
+        } else {
+            v /= 1e9;
+            unit = 'GB';
+        }
+    } else {
+        v = '?';
+        unit = '';
+    }
+    uDom.nodeFromId('storageUsed').textContent =
+        vAPI.i18n('storageUsed')
+            .replace('{{value}}', v.toLocaleString(undefined, { maximumSignificantDigits: 3 }))
+            .replace('{{unit}}', unit && vAPI.i18n(unit) || '');
 
     const timeOptions = {
         weekday: 'long',
@@ -145,28 +154,29 @@ const onLocalDataReceived = function(details) {
     const lastBackupFile = details.lastBackupFile || '';
     if ( lastBackupFile !== '' ) {
         const dt = new Date(details.lastBackupTime);
-        uDom('#localData > ul > li:nth-of-type(2) > ul > li:nth-of-type(1)').text(dt.toLocaleString('fullwide', timeOptions));
-        //uDom('#localData > ul > li:nth-of-type(2) > ul > li:nth-of-type(2)').text(lastBackupFile);
-        uDom('#localData > ul > li:nth-of-type(2)').css('display', '');
+        const text = vAPI.i18n('settingsLastBackupPrompt');
+        const node = uDom.nodeFromId('settingsLastBackupPrompt');
+        node.textContent = text + '\xA0' + dt.toLocaleString('fullwide', timeOptions);
+        node.style.display = '';
     }
 
     const lastRestoreFile = details.lastRestoreFile || '';
-    uDom('#localData > p:nth-of-type(3)');
     if ( lastRestoreFile !== '' ) {
         const dt = new Date(details.lastRestoreTime);
-        uDom('#localData > ul > li:nth-of-type(3) > ul > li:nth-of-type(1)').text(dt.toLocaleString('fullwide', timeOptions));
-        uDom('#localData > ul > li:nth-of-type(3) > ul > li:nth-of-type(2)').text(lastRestoreFile);
-        uDom('#localData > ul > li:nth-of-type(3)').css('display', '');
+        const text = vAPI.i18n('settingsLastRestorePrompt');
+        const node = uDom.nodeFromId('settingsLastRestorePrompt');
+        node.textContent = text + '\xA0' + dt.toLocaleString('fullwide', timeOptions);
+        node.style.display = '';
     }
 
     if ( details.cloudStorageSupported === false ) {
-        uDom('#cloud-storage-enabled').attr('disabled', '');
+        uDom('[data-setting-name="cloudStorageEnabled"]').attr('disabled', '');
     }
 
     if ( details.privacySettingsSupported === false ) {
-        uDom('#prefetching-disabled').attr('disabled', '');
-        uDom('#hyperlink-auditing-disabled').attr('disabled', '');
-        uDom('#webrtc-ipaddress-hidden').attr('disabled', '');
+        uDom('[data-setting-name="prefetchingDisabled"]').attr('disabled', '');
+        uDom('[data-setting-name="hyperlinkAuditingDisabled"]').attr('disabled', '');
+        uDom('[data-setting-name="webrtcIPAddressHidden"]').attr('disabled', '');
     }
 };
 
@@ -175,9 +185,10 @@ const onLocalDataReceived = function(details) {
 const resetUserData = function() {
     const msg = vAPI.i18n('aboutResetDataConfirm');
     const proceed = window.confirm(msg);
-    if ( proceed ) {
-        messaging.send('dashboard', { what: 'resetUserData' });
-    }
+    if ( proceed !== true ) { return; }
+    vAPI.messaging.send('dashboard', {
+        what: 'resetUserData',
+    });
 };
 
 /******************************************************************************/
@@ -185,21 +196,18 @@ const resetUserData = function() {
 const synchronizeDOM = function() {
     document.body.classList.toggle(
         'advancedUser',
-        uDom.nodeFromId('advanced-user-enabled').checked === true
+        uDom.nodeFromSelector('[data-setting-name="advancedUserEnabled"]').checked === true
     );
 };
 
 /******************************************************************************/
 
 const changeUserSettings = function(name, value) {
-    messaging.send(
-        'dashboard',
-        {
-            what: 'userSettings',
-            name: name,
-            value: value
-        }
-    );
+    vAPI.messaging.send('dashboard', {
+        what: 'userSettings',
+        name,
+        value,
+    });
 };
 
 /******************************************************************************/
@@ -243,7 +251,7 @@ const onUserSettingsReceived = function(details) {
                 });
     });
 
-    uDom('[data-setting-name="noLargeMedia"] ~ label:first-of-type > input[type="number"]')
+    uDom('[data-i18n="settingsNoLargeMediaPrompt"] > input[type="number"]')
         .attr('data-setting-name', 'largeMediaSize')
         .attr('data-setting-type', 'input');
 
@@ -253,7 +261,7 @@ const onUserSettingsReceived = function(details) {
              .on('click', onPreventDefault);
     });
 
-    uDom('#export').on('click', exportToFile);
+    uDom('#export').on('click', ( ) => { exportToFile(); });
     uDom('#import').on('click', startImportFilePicker);
     uDom('#reset').on('click', resetUserData);
     uDom('#restoreFilePicker').on('change', handleImportFilePicker);
@@ -263,8 +271,13 @@ const onUserSettingsReceived = function(details) {
 
 /******************************************************************************/
 
-messaging.send('dashboard', { what: 'userSettings' }, onUserSettingsReceived);
-messaging.send('dashboard', { what: 'getLocalData' }, onLocalDataReceived);
+vAPI.messaging.send('dashboard', { what: 'userSettings' }).then(result => {
+    onUserSettingsReceived(result);
+});
+
+vAPI.messaging.send('dashboard', { what: 'getLocalData' }).then(result => {
+    onLocalDataReceived(result);
+});
 
 // https://github.com/uBlockOrigin/uBlock-issues/issues/591
 document.querySelector(
