@@ -96,9 +96,16 @@ const onBeforeRequest = function(details) {
 
     // Not blocked
     if ( result !== 1 ) {
-        if ( details.parentFrameId !== -1 && details.type === 'sub_frame' ) {
+        if (
+            details.parentFrameId !== -1 &&
+            details.type === 'sub_frame' &&
+            details.aliasURL === undefined
+        ) {
             pageStore.setFrame(details.frameId, details.url);
         }
+        if ( result === 2 ) {
+            return { cancel: false };
+        }
         return;
     }
 
@@ -217,11 +224,10 @@ const onBeforeRootFrameRequest = function(fctxt) {
     if ( logData === undefined  ) { return; }
 
     // Blocked
-    const query = btoa(JSON.stringify({
+    const query = encodeURIComponent(JSON.stringify({
         url: requestURL,
         hn: requestHostname,
         dn: fctxt.getDomain() || requestHostname,
-        fc: logData.compiled,
         fs: logData.raw
     }));
 
@@ -237,9 +243,14 @@ const onBeforeRootFrameRequest = function(fctxt) {
 
 // https://github.com/gorhill/uBlock/issues/3208
 //   Mind case insensitivity.
-
+// https://github.com/uBlockOrigin/uBlock-issues/issues/1147
+//   Do not strict-block if the filter pattern does not contain at least one
+//   token character.
 const toBlockDocResult = function(url, hostname, logData) {
     if ( typeof logData.regex !== 'string' ) { return false; }
+    if ( typeof logData.raw === 'string' && /\w/.test(logData.raw) === false ) {
+        return false;
+    }
     const re = new RegExp(logData.regex, 'i');
     const match = re.exec(url.toLowerCase());
     if ( match === null ) { return false; }
@@ -248,9 +259,13 @@ const toBlockDocResult = function(url, hostname, logData) {
     // https://github.com/chrisaljoudi/uBlock/issues/1212
     //   Verify that the end of the match is anchored to the end of the
     //   hostname.
-    const end = match.index + match[0].length -
-                url.indexOf(hostname) - hostname.length;
-    return end === 0 || end === 1;
+    // https://github.com/uBlockOrigin/uAssets/issues/7619#issuecomment-653010310
+    //   Also match FQDN.
+    const hnpos = url.indexOf(hostname);
+    const hnlen = hostname.length;
+    const end = match.index + match[0].length - hnpos - hnlen;
+    return end === 0 || end === 1 ||
+           end === 2 && url.charCodeAt(hnpos + hnlen) === 0x2E /* '.' */;
 };
 
 /******************************************************************************/
@@ -448,11 +463,7 @@ const onHeadersReceived = function(details) {
     const responseHeaders = details.responseHeaders;
 
     if ( requestType === 'image' || requestType === 'media' ) {
-        return foilLargeMediaElement(
-            fctxt,
-            pageStore,
-            responseHeaders
-        );
+        return foilLargeMediaElement(fctxt, pageStore, responseHeaders);
     }
 
     if ( isDoc === false ) { return; }
@@ -844,14 +855,12 @@ const injectCSP = function(fctxt, pageStore, responseHeaders) {
 
     // Static filtering.
 
-    const logDataEntries = loggerEnabled ? [] : undefined;
-
-    µb.staticNetFilteringEngine.matchAndFetchData(
-        'csp',
-        fctxt.url,
-        cspSubsets,
-        logDataEntries
-    );
+    const staticDirectives =
+        µb.staticNetFilteringEngine.matchAndFetchData(fctxt, 'csp');
+    for ( const directive of staticDirectives ) {
+        if ( directive.result !== 1 ) { continue; }
+        cspSubsets.push(directive.getData('csp'));
+    }
 
     // URL filtering `allow` rules override static filtering.
     if (
@@ -893,10 +902,11 @@ const injectCSP = function(fctxt, pageStore, responseHeaders) {
     // <<<<<<<< All policies have been collected
 
     // Static CSP policies will be applied.
-    if ( logDataEntries !== undefined ) {
+
+    if ( loggerEnabled && staticDirectives.length !== 0 ) {
         fctxt.setRealm('network').setType('csp');
-        for ( const entry of logDataEntries ) {
-            fctxt.setFilter(entry).toLogger();
+        for ( const directive of staticDirectives ) {
+            fctxt.setFilter(directive.logData()).toLogger();
         }
     }
 
@@ -1034,15 +1044,7 @@ const strictBlockBypasser = {
 return {
     start: (( ) => {
         vAPI.net = new vAPI.Net();
-
-        if (
-            vAPI.net.canSuspend() &&
-            µBlock.hiddenSettings.suspendTabsUntilReady !== 'no' ||
-            vAPI.net.canSuspend() !== true &&
-            µBlock.hiddenSettings.suspendTabsUntilReady === 'yes'
-        ) {
-            vAPI.net.suspend(true);
-        }
+        vAPI.net.suspend();
 
         return function() {
             vAPI.net.setSuspendableListener(onBeforeRequest);
@@ -1072,7 +1074,7 @@ return {
                     [ 'blocking', 'requestBody' ]
                 );
             }
-            vAPI.net.unsuspend();
+            vAPI.net.unsuspend(true);
         };
     })(),
 
