@@ -11,25 +11,12 @@ mkdir -p $DES
 echo "*** uBlock0.opera: copying common files"
 bash ./tools/copy-common-files.sh  $DES
 
-echo "*** uBlock0.opera: concatenating content scripts"
-cat $DES/js/vapi-usercss.js > /tmp/contentscript.js
-echo >> /tmp/contentscript.js
-grep -v "^'use strict';$" $DES/js/vapi-usercss.real.js >> /tmp/contentscript.js
-echo >> /tmp/contentscript.js
-grep -v "^'use strict';$" $DES/js/vapi-usercss.pseudo.js >> /tmp/contentscript.js
-echo >> /tmp/contentscript.js
-grep -v "^'use strict';$" $DES/js/contentscript.js >> /tmp/contentscript.js
-mv /tmp/contentscript.js $DES/js/contentscript.js
-rm $DES/js/vapi-usercss.js
-rm $DES/js/vapi-usercss.real.js
-rm $DES/js/vapi-usercss.pseudo.js
-
 # Opera-specific
 cp platform/opera/manifest.json $DES/
 rm -r $DES/_locales/az
-rm -r $DES/_locales/bs
 rm -r $DES/_locales/cv
 rm -r $DES/_locales/hi
+rm -r $DES/_locales/hy
 rm -r $DES/_locales/ka
 rm -r $DES/_locales/kk
 rm -r $DES/_locales/mr
